# Generated by Django 5.0.7 on 2024-07-18 16:03

import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Tenant',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created_at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated_at')),
                ('deleted_at', models.DateTimeField(blank=True, null=True, verbose_name='deleted_at')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('subdomain', models.CharField(max_length=50, unique=True)),
                ('database_name', models.CharField(max_length=50, unique=True)),
            ],
            options={
                'abstract': False,
                'swappable': 'TENANT_MODEL',
            },
        ),
    ]
